#!/usr/bin/env python
# encoding: utf-8

import os
import shutil
import logging
import tempfile

import unittest

from . import assets
from clickgen.template import __main__ as template


class TestTemplate(unittest.TestCase):

    # setup
    def setUp(self):
        logging.disable(logging.CRITICAL)

        self.mock_dir = tempfile.mkdtemp()
        self.mock_cursor_file_path = os.path.join(self.mock_dir,
                                                  'cursor.theme')
        self.mock_index_file_path = os.path.join(self.mock_dir, 'index.theme')
        self.mock_name = 'foo'
        self.mock_comment = 'foo bar'

    def tearDown(self):
        logging.disable(logging.NOTSET)

        shutil.rmtree(self.mock_dir)

    # helpers
    def assert_word_in_file(self, file: str, word: str):
        with open(file, 'r') as f:
            content = f.read()
            self.assertTrue(word in content)

    # tests
    def test_have_templates_files(self):
        self.assertTrue(os.path.exists(template.cursor_file_path))
        self.assertTrue(os.path.exists(template.index_file_path))

    def test_create_x11_template_without_comments(self):
        template.create_x11_template(name=self.mock_name,
                                     directory=self.mock_dir,
                                     comment='')
        # testing cursor.theme
        self.assert_word_in_file(file=self.mock_cursor_file_path,
                                 word='Name=foo')

        # testing index.theme
        self.assert_word_in_file(file=self.mock_index_file_path,
                                 word='Name=foo')
        self.assert_word_in_file(file=self.mock_index_file_path,
                                 word='Comment=foo cursor theme')

    def test_create_x11_template_with_comments(self):
        template.create_x11_template(name=self.mock_name,
                                     directory=self.mock_dir,
                                     comment=self.mock_comment)
        # testing cursor.theme
        self.assert_word_in_file(file=self.mock_cursor_file_path,
                                 word='Name=foo')

        # testing index.theme
        self.assert_word_in_file(file=self.mock_index_file_path,
                                 word='Name=foo')
        self.assert_word_in_file(file=self.mock_index_file_path,
                                 word='Comment=foo bar')


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
# encoding: utf-8

import os

from .__main__ import main
from .configsgen.__main__ import main as configsgen
from .configsgen.__main__ import DELAY

from .types import Path, IntegerList
from .helpers import TemporaryDirectory


def build_win_cursor_theme(name: str, image_dir: Path, cursor_sizes: IntegerList, hotspots: any = None, out_path: Path = os.getcwd(), archive: bool = False, delay: int = DELAY):
    """
    Build Functions for Windows Cursor Theme.
    `name` is folder name.
    `image_dir` is where all images are stored.Cursor naming is based on `image name`.
    `cursor_sizes` is pixel size for generating cursor(basically in `ineteger array`)
    `hotspots` is `JSON` data each cursor have `xhot` and `yhot` member.
     example:
        {
            "all_scroll": {
                "xhot":2
                "yhot":4
            }
        }
    """

    # generate configs to temporary directory
    with TemporaryDirectory() as configs:
        configsgen(image_dir, cursor_sizes, hotspots,
                   out_dir=configs, delay=delay)

        main(name=name, config_dir=configs, out_path=out_path,
             x11=False, win=True, archive=archive, logs=True)


def build_x11_cursor_theme(name: str, image_dir: Path, cursor_sizes: IntegerList, hotspots: any = None, out_path: Path = os.getcwd(), archive: bool = False, delay: int = DELAY):
    """
    Build Functions for Windows Cursor Theme.
    `name` is folder name.
    `image_dir` is where all images are stored.Cursor naming is based on `image name`.
    `cursor_sizes` is pixel size for generating cursor(basically in `ineteger array`)
    `hotspots` is `JSON` data each cursor have `xhot` and `yhot` member.
     example:
        {
            "all_scroll": {
                "xhot":2
                "yhot":4
            }
        }
    """

    # generate configs to temporary directory
    with TemporaryDirectory() as configs:
        configsgen(image_dir, cursor_sizes, hotspots,
                   out_dir=configs, delay=delay)

        main(name=name, config_dir=configs, out_path=out_path,
             x11=True, win=False, archive=archive, logs=True)


def build_cursor_theme(name: str, image_dir: Path, cursor_sizes: IntegerList, hotspots: any = None, out_path: Path = os.getcwd(), archive: bool = False, delay: int = DELAY):
    """
    Build Functions for Windows Cursor Theme.
    `name` is folder name.
    `image_dir` is where all images are stored.Cursor naming is based on `image name`.
    `cursor_sizes` is pixel size for generating cursor(basically in `ineteger array`)
    `hotspots` is `JSON` data each cursor have `xhot` and `yhot` member.
     example:
        {
            "all_scroll": {
                "xhot":2
                "yhot":4
            }
        }
    """

    # generate configs to temporary directory
    with TemporaryDirectory() as configs:
        configsgen(image_dir, cursor_sizes, hotspots,
                   out_dir=configs, delay=delay)

        # build package
        main(name=name, config_dir=configs, out_path=out_path,
             x11=True, win=True, archive=archive, logs=True)



#!/usr/bin/env python
# encoding: utf-8

import sys
import glob
import logging
import os
import shutil

from .linker import link_cursors
from .template import create_x11_template
from .win import main as win_gen
from .x11 import main as x11_gen
from .configsgen.__main__ import main as configsgen

from .helpers import TemporaryDirectory, create_dir, get_logger

from .types import Path

# config variables
config_ext = ('*.in', '*.ini')
# Note: 'zip' formate remove symbolic links,replace with normal files.That can affect the archive size.
archive_format = 'tar'


def get_configs(directory: Path) -> list:
    """
        To get all config_files name with extension in Directory.
        'dir' is where config_files stored.
    """
    configs_grabbed = []

    for ext in config_ext:
        configs_grabbed.extend(
            glob.glob(os.path.abspath(os.path.join(directory, ext))))
    return configs_grabbed


def is_animated(config: Path) -> bool:
    """
       check config_file have animation or not.
       'config' is name of .in or .ini config file.
       config_file contains following information for each size:

        <size> <xhot> <yhot> <path_to_png> <delay>
            ^      ^      ^         ^          ^
            |      |      |         |          |
        column1 column2 column3  column4    colums5

        <delay> is optional information, This function is designed only for read first line of config. 
    """

    try:
        with open(config, 'r') as config:
            line = config.readline()
            columns = line.split(" ")

            # animated cursor config have 5 columns,or more
            if (len(columns) > 4):
                return True
            else:
                return False
    except IOError as err:
        print('Error: ', err)


def get_cur_name(config: Path, cur_type: str) -> str:
    """
        Get the cursor's extension by providing `cur_type` to this function.

        window extension => .cur , .ani
            watch.ani, left_ptr.cur

        x11 extension => `null`
            watch, left_ptr
    """
    config_name = os.path.basename(config)
    animated = is_animated(config)

    if (cur_type == 'win' and animated == False):
        return config_name.replace('.in', '.cur')
    elif (cur_type == 'win' and animated == True):
        return config_name.replace('.in', '.ani')
    elif (cur_type == 'x11'):
        return config_name.replace('.in', '')
    else:
        return ""


def main(name: str,
         config_dir: Path,
         out_path: Path = os.getcwd(),
         x11: bool = False,
         win: bool = False,
         archive: bool = False,
         logs: bool = False) -> None:
    """
        Generate `Window` or `X11` cursor package from configs.
        `name` is the Display Name of the cursor. That reflects a gnome-tweak-tool, KDE-settings,.. etc. In 'Window' is doesn't matter but it reflects an archive name.
        `config_dir` is where all config files stored, only `.in` or `.ini` config files supported.sample config_files => <https://github.com/ubuntu/yaru/tree/master/icons/src/cursors/bitmaps>
        `out_path` is where cursor archive or directory stored.absolute or relative path both eligible.
        `x11` & `win` are platforms flags.Using these flag to output package platforms,like `win` for "Window OS" and `x11` for "FreeDesktop" that used in "Linux".Default both flags are "False".
        `archive` flag to compress cursor `package to tar`.In python zip compression method symbolic links replace by normal file, that increase size of the output file.
        `logs` is for extra information
    """

    # logs disabled default
    logger = get_logger('clickgen')
    logging.disable(logging.CRITICAL)
    # enabling logs
    if (logs):
        logging.disable(logging.NOTSET)

    # If platforms  flags disabled
    try:
        if (x11 == False and win == False):
            raise ValueError('🚨 Cursor generate type missing')

    except ValueError as valerr:
        print('Error:', valerr)
        sys.exit(1)

    # passing configs path
    in_path = os.path.abspath(config_dir)

    try:
        configs = get_configs(config_dir)
        if (len(configs) <= 0):
            raise FileNotFoundError('🚨 No configs found in %s' % in_path)

    except FileNotFoundError as err:
        print(err)
        sys.exit(1)

    out = os.path.abspath(os.path.join(out_path, name))

    # set png prefix
    prefix = os.path.abspath(config_dir)

    # generate cursor based on flags
    if (win):
        print('👷 Building Windows cursors..')

        with TemporaryDirectory() as win_work_dir:

            for config in configs:
                cur_name = get_cur_name(config, cur_type='win')
                cur_out = os.path.join(win_work_dir, cur_name)
                win_gen(input_config=config,
                        output_file=cur_out,
                        prefix=prefix)
                logger.info('Window: cursor generated to %s' % cur_out)

            link_cursors(win_work_dir, win=True)
            win_out = os.path.join(out, 'win')

            # deleting existed win directory
            if (os.path.exists(win_out)):
                shutil.rmtree(win_out)

            shutil.copytree(win_work_dir, win_out)

    if (x11):
        print('👷 Building X11 cursors..')

        with TemporaryDirectory() as x11_work_dir:
            x11_cursors_dir = os.path.join(x11_work_dir, 'cursors')
            create_dir(x11_cursors_dir)

            for config in configs:
                cur_name = get_cur_name(config, cur_type='x11')

                cur_out = os.path.join(x11_cursors_dir, cur_name)
                x11_gen(input_config=config,
                        output_file=cur_out,
                        prefix=prefix)

                logger.info('X11: cursor generated to %s' % cur_out)

            link_cursors(x11_cursors_dir)
            create_x11_template(name=name, directory=x11_work_dir)
            x11_out = os.path.join(out, 'x11')

            # deleting existed x11 directory
            if (os.path.exists(x11_out)):
                shutil.rmtree(x11_out)

            shutil.copytree(x11_work_dir, x11_out, symlinks=True)

    if (archive):
        shutil.make_archive(out, archive_format, out)
        shutil.rmtree(out)



#!/usr/bin/env python
# encoding: utf-8

import ctypes
import os

from .types import Path

basedir: Path = os.path.abspath(os.path.dirname(__file__))
libpath: Path = os.path.join(basedir, 'xcursorgen.so')

dll: Path = ctypes.CDLL(libpath)

# main function ctypes define
LP_c_char = ctypes.POINTER(ctypes.c_char)
LP_LP_c_char = ctypes.POINTER(LP_c_char)

dll.main.argtypes = (ctypes.c_int, LP_LP_c_char)


def gen_argv_ctypes(argv: list) -> LP_LP_c_char:
    p = (LP_c_char * len(argv))()

    for i, arg in enumerate(argv):  # not sys.argv, but argv!!!
        enc_arg = str(arg).encode('utf-8')
        p[i] = ctypes.create_string_buffer(enc_arg)

    return ctypes.cast(p, LP_LP_c_char)


def generate(argc: int, argv: list) -> None:
    """
        Call to xcursorgen.c 'main' function.
        'argc' is length of arguments,i.e: length of list.
        'argv' is list of arguments. Each command-line argument separated by ' '(Space) in C, but here by 'elements' of list.
        for example:

        In Terminal:
            ~$ xcursorgen -v

        In list:
            ['xcursorgen','-v']
            ['xcursorgen','-p','<png_dir>','<path_to_input_config>','<path_to_output_file>']

        In list binary_name as the first element is mandatory for executing the binary here is 'xcursorgen'.
    """
    na = gen_argv_ctypes(argv)
    dll.main(argc, na)


def main(input_config: Path, output_file: Path, prefix: Path) -> None:
    """
         xcursorgen.c python api
        'input_config' is path to config_file.
        'output_file' is a path to store process cursor.
        In 'input_config' & 'output_file' absolute or relative both aceptable.
        'prefix' is a path to '.png files' link in the config_file, if relative path implemented.
    """
    # binary name as first argument
    argv = ['xcursorgen', '-p', prefix, input_config, output_file]
    argc = len(argv)

    generate(argc, argv)



#!/usr/bin/env python
# encoding: utf-8

import os
import shutil
import tempfile
import unittest

from . import assets
from clickgen import win


class TestWinBuilder(unittest.TestCase):

    # setup
    def setUp(self):
        self.temp_dir = tempfile.mkdtemp()
        self.mock_prefix = assets.mock_config_path
        self.mock_static_out = os.path.join(self.temp_dir, 'mock_static.cur')
        self.mock_animated_out = os.path.join(self.temp_dir,
                                              'mock_animated.ani')

    def tearDown(self):
        shutil.rmtree(self.temp_dir)

    # helpers
    def assert_cursor_size(self, path: str):
        self.assertGreater(os.path.getsize(path), 0)

    # tests
    # TODO: More to test here
    def test_main(self):
        # testing static cursor
        win.main(input_config=assets.static_mock_config_path,
                 output_file=self.mock_static_out,
                 prefix=self.mock_prefix)
        self.assert_cursor_size(self.mock_static_out)

        # testing animated cursor
        win.main(input_config=assets.animated_mock_config_path,
                 output_file=self.mock_animated_out,
                 prefix=self.mock_prefix)
        self.assert_cursor_size(self.mock_animated_out)


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
# encoding: utf-8

import os
import logging

import unittest

from clickgen.linker import __main__ as linker


class TestLinker(unittest.TestCase):

    # setup
    def setUp(self):
        logging.disable(logging.CRITICAL)
        self.mock_directory = ['foo', 'bar', 'zoo']

    def tearDown(self):
        logging.disable(logging.NOTSET)

    # helpers
    def assert_to_directory(self, name: str, expect_name: str):
        mock_result = linker.match_to_directory(name,
                                                directory=self.mock_directory)
        self.assertEqual(mock_result, expect_name)

    # tests
    def test_match_to_directory(self):
        # word is similar to directory
        self.assert_to_directory(name='F0o', expect_name='foo')
        # word new to directory
        self.assert_to_directory(name='foo_bar_zoo', expect_name='foo_bar_zoo')

    def test_load_data(self):
        # checking data file extists
        self.assertTrue(os.path.exists(linker.data_file))


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python
# encoding: utf-8

import pathlib
from logging import Logger
from typing import List, Union, Tuple

Path = Union[str, pathlib.Path]

StringList = List[str]

IntegerList = List[int]

IntegerTuple = Tuple[int, int]
CoordinateTuple = Union[Tuple[int, int], None]



#!/usr/bin/env python
# encoding: utf-8

import os

from ..helpers import create_dir, get_logger
from ..types import Path, Logger

# files path
basedir: Path = os.path.abspath(os.path.dirname(__file__))
cursor_file_path: Path = os.path.join(basedir, 'cursor.theme')
index_file_path: Path = os.path.join(basedir, 'index.theme')

# Logger
logger: Logger = get_logger('clickgen:template')


def create_x11_template(directory: Path, name: Path, comment: str = '') -> None:
    """
        Copy metadata files to the cursor package directory.
        `directory` is a path to the cursor package directory.
        `name` & `comment` is Metadata for cursor package.
    """
    abs_dir = os.path.abspath(directory)
    cursor_file_out_path = os.path.join(abs_dir, 'cursor.theme')
    index_file_out_path = os.path.join(abs_dir, 'index.theme')

    # helper
    create_dir(directory)

    with open(cursor_file_path, 'r') as cursor_file:
        with open(cursor_file_out_path, 'w') as cursor_file_out:
            lines = cursor_file.readlines()

            for line in lines:
                cursor_file_out.write(line.replace('<Name>', name))

            logger.info('cursor.theme file generated for %s at %s' %
                        (name, cursor_file_out_path))

    with open(index_file_path, 'r') as index_file:
        with open(index_file_out_path, 'w') as index_file_out:
            lines = index_file.readlines()

            for line in lines:
                temp_line = line.replace('<Name>', name)

                if (comment != ''):
                    temp_line = temp_line.replace('<Comment>', comment)
                else:
                    temp_line = temp_line.replace('<Comment>',
                                                  '%s cursor theme' % name)

                index_file_out.write(temp_line)

            logger.info('index.theme file generated for %s at %s' %
                        (name, index_file_out_path))



#!/usr/bin/env python
# encoding: utf-8

import os
import itertools
from PIL import Image

from ..helpers import get_logger
from ..types import Path, IntegerList, StringList, IntegerTuple, CoordinateTuple, Logger


# Logger
logger: Logger = get_logger('clickgen:configsgen')

# Default configs path is <working_dir>/configs/
DEFAULT_CONFIGS_PATH = os.path.join(os.getcwd(), 'configs')

# SET DELAY OF ANIMATED FRAMES IN CONFIG FILE
DELAY = 20


def get_cursor_list(imgs_dir: Path, animated: bool = False) -> StringList:
    """
        Get cursors from `imgs_dir` based on `animated` flag.
        `animated` flag is usefull for get animated cursors frames, Frames per cursor is identify by `number(01, 02, .., n)` with `postfix` of cursor name like `wait-01.png`.
    """

    all_cursor_list, cursor_list = [], []

    for file_path in os.listdir(imgs_dir):
        all_cursor_list.append(os.path.basename(file_path))

    if (animated):
        # animated cursor have filename-01,02,03..n postfix
        temp: StringList = [cursor for cursor in all_cursor_list if
                            cursor.find("-") >= 0]
        temp.sort()
        cursor_list: StringList = [list(g) for _, g in itertools.groupby(
            temp, lambda x: x.partition("-")[0])]
    else:
        for cursor in all_cursor_list:
            if cursor.find("-") <= 0:
                cursor_list.append(cursor)
        cursor_list.sort()

    return cursor_list


def resize_cursor(cursor: str, size: int, imgs_dir: Path, coordinates: CoordinateTuple, out_dir: Path = DEFAULT_CONFIGS_PATH) -> IntegerTuple:
    """
        `cursor` is cursor `name`.
        `size` is pixel size of cursor to resize.
        `imgs_dir` is where `cursor` is found.
        `coordinares` is `InegerTuple` contains `(xhot, yhot)`.
        `out_dir` is `Path` for resized cursor, `default` set to `configs` directory inside `workdir()`.
    """

    # helper variables
    in_path = os.path.join(imgs_dir, cursor)
    out_dir = os.path.join(out_dir, '%sx%s' % (size, size))
    out_path = os.path.join(out_dir, cursor)

    if not os.path.exists(out_dir):
        os.makedirs(out_dir)

    # opening original image
    image = Image.open(in_path)

    width = image.size[0]
    height = image.size[1]

    aspect = width / float(height)

    ideal_width = size
    ideal_height = size

    ideal_aspect = ideal_width / float(ideal_height)

    if aspect > ideal_aspect:
        # Then crop the left and right edges:
        new_width = int(ideal_aspect * height)
        offset = (width - new_width) / 2
        resize = (offset, 0, width - offset, height)
    else:
        # ... crop the top and bottom:
        new_height = int(width / ideal_aspect)
        offset = (height - new_height) / 2
        resize = (0, offset, width, height-offset)

    thumb = image.crop(resize).resize(
        (ideal_width, ideal_height), Image.ANTIALIAS)

    # save resized image
    thumb.save(out_path)

    # closing PIL.Image instances
    image.close()
    thumb.close()

    #  finding new X & Y coordinates
    if coordinates is None:
        Rx = Ry = int(size / 2)
    else:
        (xhot, yhot) = coordinates
        Rx = round(size / width * xhot)
        Ry = round(size / height * yhot)

    return (Rx, Ry)


def write_xcur(config_file_path: Path, content: StringList) -> None:
    """
        Write `content` to `config_file_path`.
        config file extension is `.in` or `.ini`.
    """

    # sort line, So all lines in order according to size (24x24, 28x28, ..)
    content.sort()

    # remove newline end of file `xcursorgen` generate error
    content[-1] = content[-1].rstrip("\n")

    with open(config_file_path, "w") as config_file:
        for line in content:
            config_file.write(line)
        config_file.close()


def generate_static_cursor(imgs_dir: Path, out_dir: Path, sizes: IntegerList, hotspots: any) -> None:
    """
        Generate Staic cursor & config.
        `imgs_dir` is `directory` where images are stored.
        `out_dir` is `directory` for storing generated resized images & config files.
        `sizes` is `List` of pixel size.
        `hotspots` is `JSON` data each cursor have `xhot` and `yhot` member.
        example:
            {
                "all_scroll": {
                    "xhot":2
                    "yhot":4
                }
            }
    """

    cursors_list: StringList = get_cursor_list(imgs_dir)

    for cursor in cursors_list:

        config_file_path: Path = os.path.join(
            out_dir, cursor.replace(".png", ".in"))

        content: StringList = []

        # setting Hotspots from JSON data
        # set to `None` if not have JSON data or `key` not in JSON
        cursor_name = cursor.split('.')[0]
        coordinate: CoordinateTuple = None

        try:
            hotspot = hotspots[cursor_name]
            xhot: int = hotspot['xhot']
            yhot: int = hotspot['yhot']
            coordinate = (xhot, yhot)
        except TypeError:
            coordinate = None

        for size in sizes:
            resized_xhot, resized_yhot = resize_cursor(
                cursor, size, imgs_dir, coordinate, out_dir=out_dir)

            logger.info('%sx%s %s hotspots set to (%s,%s)' %
                        (size, size, cursor_name, resized_xhot, resized_yhot))

            line = "%s %s %s %sx%s/%s\n" % (size,
                                            resized_xhot, resized_yhot, size, size, cursor)
            content.append(line)

        write_xcur(config_file_path, content)


def generate_animated_cursor(imgs_dir: Path, out_dir: Path, sizes: IntegerList, hotspots: any, delay: int = DELAY):
    """
        Generate Animated cursor & config.
        `imgs_dir` is `directory` where images are stored.
        `out_dir` is `directory` for storing generated resized images & config files.
        `sizes` is `List` of pixel size.
        `hotspots` is `JSON` data each cursor have `xhot` and `yhot` member.
        example:
            {
                "wait": {
                    "xhot":2
                    "yhot":4
                }
            }
    """

    cursors_list: StringList = get_cursor_list(imgs_dir, animated=True)

    for group in cursors_list:
        group_name = str(group[0]).split("-")[0]
        config_file_path: Path = os.path.join(out_dir, group_name + ".in")

        content: StringList = []

        # setting Hotspots from JSON data
        # set to `None` if not have JSON data or `key` not in JSON
        coordinate: CoordinateTuple = None

        try:
            hotspot = hotspots[group_name]
            xhot = hotspot['xhot']
            yhot = hotspot['yhot']
            coordinate = (xhot, yhot)
        except TypeError:
            coordinate = None

        for cursor in group:

            for size in sizes:
                resized_xhot, resized_yhot = resize_cursor(
                    cursor, size, imgs_dir, coordinate, out_dir=out_dir)

                logger.info('%sx%s %s hotspots set to (%s,%s) with %sms delay' %
                            (size, size, group_name, resized_xhot, resized_yhot, delay))

                line = "%s %s %s %sx%s/%s %s\n" % (size,
                                                   resized_xhot, resized_yhot, size, size, cursor, delay)
                content.append(line)

        write_xcur(config_file_path, content)


def main(imgs_dir: Path, cursor_sizes: IntegerList, hotspots: any, out_dir: Path = DEFAULT_CONFIGS_PATH, delay: int = DELAY) -> Path:
    """
        Generate Configs files.
        hotspots is JSON data for each cursor having xhot & yhot parameters.
        Provide `None` value set hotspots to middle of cursor.
        hotspots is default set to `None`
    """

    logger.info('configs are writen to %s' % out_dir)

    generate_static_cursor(imgs_dir, sizes=cursor_sizes,
                           hotspots=hotspots, out_dir=out_dir)
    generate_animated_cursor(imgs_dir, sizes=cursor_sizes,
                             hotspots=hotspots, out_dir=out_dir, delay=delay)

    return (os.path.abspath(out_dir))



#!/usr/bin/env python
# encoding: utf-8

import os
import shutil
import tempfile
import unittest

from . import assets
from clickgen import x11
from clickgen.x11 import libpath


class TestX11Builder(unittest.TestCase):

    # setup
    def setUp(self):
        self.mock_argv = ['foo', 'bar']
        self.temp_dir = tempfile.mkdtemp()
        self.mock_prefix = assets.mock_config_path
        self.mock_static_out = os.path.join(self.temp_dir, 'mock_static')
        self.mock_animated_out = os.path.join(self.temp_dir, 'mock_animated')

    def tearDown(self):
        shutil.rmtree(self.temp_dir)

    # helpers
    def assert_cursor_size(self, path: str):
        self.assertGreater(os.path.getsize(path), 0)

    # tests
    def test_gen_argv_ctypes(self):
        # testing return proper argument class
        mock_result = x11.gen_argv_ctypes(self.mock_argv)
        self.assertEqual(str(mock_result.__class__),
                         "<class 'clickgen.x11.LP_LP_c_char'>")

    def test_generate(self):
        # binary exists
        self.assertTrue(os.path.exists(libpath))

    def test_main(self):
        # testing static cursor
        x11.main(input_config=assets.static_mock_config_path,
                 output_file=self.mock_static_out,
                 prefix=self.mock_prefix)
        self.assert_cursor_size(self.mock_static_out)

        # testing animated cursor
        x11.main(input_config=assets.animated_mock_config_path,
                 output_file=self.mock_animated_out,
                 prefix=self.mock_prefix)
        self.assert_cursor_size(self.mock_animated_out)


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
# encoding: utf-8

import os
import shutil
import tempfile

import unittest

from clickgen import __main__ as clickgen
from . import assets


class TestMain(unittest.TestCase):

    # setup
    def setUp(self):
        self.mock_config_dir_with_configs = assets.mock_config_path
        self.mock_config_dir_with_out_configs = tempfile.mkdtemp()
        self.mock_config_path = assets.mock_config_path
        self.mock_out_path = tempfile.mkdtemp()
        self.mock_name = 'foo'
        self.foo_dir_path = os.path.join(self.mock_out_path, 'foo')
        self.foo_archive_path = os.path.join(self.mock_out_path, 'foo.tar')

    def tearDown(self):
        shutil.rmtree(self.mock_config_dir_with_out_configs)
        shutil.rmtree(self.mock_out_path)

    # helpers
    def assert_cur_name(self, config: str, type: str, expect_ext: str):
        ext = clickgen.get_cur_name(config, type)

        self.assertEqual(ext, expect_ext)

    def assert_win_extension(self, extension: str):
        return True if extension == '.cur' or extension == '.ani' else False

    def assert_x11_dir(self):
        self.assertTrue(os.path.exists(self.foo_dir_path))

        mock_x11_dir = os.path.join(self.foo_dir_path, 'x11')
        self.assertTrue(os.path.exists(mock_x11_dir))

        expect_dir_struc = sorted(['cursor.theme', 'cursors', 'index.theme'])
        result_dir_struc = sorted(os.listdir(mock_x11_dir))
        self.assertEqual(len(result_dir_struc), len(expect_dir_struc))
        self.assertListEqual(result_dir_struc, expect_dir_struc)

        mock_cursor_dir = os.path.join(mock_x11_dir, 'cursors')
        self.assertTrue(os.path.exists(mock_cursor_dir))
        self.assertGreater(len(os.listdir(mock_cursor_dir)), 0)

        for cursor in os.listdir(mock_cursor_dir):
            _, extension = os.path.splitext(cursor)
            self.assertEqual(extension, '')

    def assert_win_dir(self):

        self.assertTrue(os.path.exists(self.foo_dir_path))

        mock_win_dir = os.path.join(self.foo_dir_path, 'win')
        self.assertTrue(os.path.exists(mock_win_dir))
        self.assertGreater(len(os.listdir(mock_win_dir)), 0)

        for cursor in os.listdir(mock_win_dir):
            _, extension = os.path.splitext(cursor)
            self.assert_win_extension(extension)

    # test
    def test_get_config(self):
        # testing with assets dir,have 2 config files
        mock_configs = clickgen.get_configs(
            directory=self.mock_config_dir_with_configs)
        self.assertEqual(len(mock_configs), 2)

        # testing with empty dir
        mock_configs = clickgen.get_configs(
            directory=self.mock_config_dir_with_out_configs)
        self.assertEqual(len(mock_configs), 0)

    def test_is_animated(self):
        # testing config file with aimation
        self.assertTrue(
            clickgen.is_animated(
                config=assets.animated_mock_config_path))

        # testing config file without aimation
        self.assertFalse(
            clickgen.is_animated(config=assets.static_mock_config_path))

    def test_get_cur_name(self):
        # window extensions
        self.assert_cur_name(config=assets.animated_mock_config_path,
                             type='win',
                             expect_ext='mock_animated.ani')
        self.assert_cur_name(config=assets.static_mock_config_path,
                             type='win',
                             expect_ext='mock_static.cur')

        # x11 extensions
        self.assert_cur_name(config=assets.animated_mock_config_path,
                             type='x11',
                             expect_ext='mock_animated')
        self.assert_cur_name(config=assets.static_mock_config_path,
                             type='x11',
                             expect_ext='mock_static')

    def test_x11_dir_struc(self):
        # testing x11 with directory
        clickgen.main(name=self.mock_name,
                      config_dir=self.mock_config_path,
                      out_path=self.mock_out_path,
                      x11=True)

        self.assert_x11_dir()

    def test_win_dir_struc(self):
        # testing win with directory
        clickgen.main(name=self.mock_name,
                      config_dir=self.mock_config_path,
                      out_path=self.mock_out_path,
                      win=True)

        self.assert_win_dir()

    def test_dir_struc(self):
        clickgen.main(name=self.mock_name,
                      config_dir=self.mock_config_path,
                      out_path=self.mock_out_path,
                      win=True,
                      x11=True)

        self.assert_win_dir()
        self.assert_x11_dir()

    def test_archive(self):
        clickgen.main(name=self.mock_name,
                      config_dir=self.mock_config_path,
                      out_path=self.mock_out_path,
                      win=True,
                      x11=True,
                      archive=True)

        mock_out_file = os.path.join(self.mock_out_path,
                                     self.mock_name + '.tar')

        self.assertGreater(os.path.getsize(mock_out_file), 0)


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python
# encoding: utf-8

import unittest
import tempfile
import shutil
import logging
import filecmp
import os
from PIL import Image

from . import assets
from clickgen import configsgen


class TestConfigsgen(unittest.TestCase):

    # setup
    def setUp(self):
        logging.disable(logging.CRITICAL)

        self.temp_dir = tempfile.mkdtemp()

        # for test_resize_image
        self.mock_cursor = 'mock_static.png'
        self.mock_size = 100
        self.mock_coordinates = (47, 25)
        self.mock_images_path = assets.mock_images_path
        self.resulted_resize_image_path = os.path.join(
            self.temp_dir, '%sx%s' % (self.mock_size, self.mock_size), self.mock_cursor)

        # for test_write_xcur
        self.mock_config_path = os.path.join(self.temp_dir, 'foo.in')
        self.mock_config_content = ['foo\n', 'bar\n', 'zoo\n']

        # for test_generate_static_cursor
        # 📝 Note : doen't change size that Fail Some test
        self.mock_sizes = [50, 28]
        self.mock_hotspots = assets.get_mock_hotspots()

    def tearDown(self):
        shutil.rmtree(self.temp_dir)
        logging.disable(logging.NOTSET)

    # helpers
    def assert_file_is_valid(self, path: str):
        self.assertTrue(os.path.isfile(path))
        self.assertGreater(os.path.getsize(path), 0)

    def assert_image_size(self, img_path: str, size: int):
        mock_resize_image_instace = Image.open(img_path)

        self.assertTupleEqual(mock_resize_image_instace.size,
                              (size, size))
        self.assertEqual(mock_resize_image_instace.mode, 'RGBA')

        mock_resize_image_instace.close()

    # tests
    def test_get_cursor_list(self):
        # testing static images list
        mock_static_list = configsgen.get_cursor_list(
            imgs_dir=assets.mock_images_path, animated=False)

        self.assertEqual(mock_static_list, ['mock_static.png'])

        # testing animated images list
        mock_animated_list = configsgen.get_cursor_list(
            imgs_dir=assets.mock_images_path, animated=True)

        self.assertEqual(mock_animated_list, [['mock_animated_1-01.png', 'mock_animated_1-02.png'], [
            'mock_animated_2-01.png', 'mock_animated_2-02.png']])

    def test_resize_cursor(self):
        # 🧪 TEST FOR COORDINATORS
        result_coordinates_tuple = configsgen.resize_cursor(
            cursor=self.mock_cursor, size=self.mock_size, imgs_dir=self.mock_images_path, coordinates=self.mock_coordinates, out_dir=self.temp_dir)

        # testing coordinates result
        self.assertTupleEqual(result_coordinates_tuple, (24, 12))

        # 🧪 TEST FOR `NONE` COORDINATORS
        result_coordinates_tuple = configsgen.resize_cursor(
            cursor=self.mock_cursor, size=self.mock_size, imgs_dir=self.mock_images_path, coordinates=None, out_dir=self.temp_dir)

        # testing coordinates result
        self.assertTupleEqual(result_coordinates_tuple,
                              (self.mock_size / 2, self.mock_size / 2))

        # test resized images exists or not
        self.assert_file_is_valid(self.resulted_resize_image_path)

        # test resized image dimensions & type as `.png`
        self.assert_image_size(
            img_path=self.resulted_resize_image_path, size=self.mock_size)

    def test_write_xcur(self):
        configsgen.write_xcur(
            config_file_path=self.mock_config_path, content=self.mock_config_content)

        self.assert_file_is_valid(self.mock_config_path)

        # testing config file content are sorted & '\n' trimdown from a last line
        with open(self.mock_config_path, 'r') as mock_config:
            mock_content = mock_config.readlines()
            self.assertListEqual(mock_content, ['bar\n', 'foo\n', 'zoo'])

    def test_generate_static_cursor(self):
        # test with `None` Hotspots
        configsgen.generate_static_cursor(
            imgs_dir=self.mock_images_path, out_dir=self.temp_dir, sizes=self.mock_sizes, hotspots=None)

        # testing generated sized directory structure
        for size in self.mock_sizes:
            # have size direcory
            size_dir = '%sx%s' % (size, size)
            self.assertIn(size_dir, os.listdir(self.temp_dir))

            size_dir_path = os.path.join(self.temp_dir, size_dir)

            images_list: [str] = assets.get_mock_static_images_list()

            # have resized images
            self.assertListEqual(images_list, os.listdir(size_dir_path))

            # test resized image dimensions & type as `.png`
            for image in images_list:
                img_path = os.path.join(size_dir_path, image)
                self.assert_image_size(img_path=img_path, size=size)

        # testing .in config file
        result_config_path = os.path.join(self.temp_dir, 'mock_static.in')
        with open(result_config_path, 'r') as file:
            content = file.read()

            for size in self.mock_sizes:
                expect_line: str = '%s %s %s %sx%s/mock_static.png' % (
                    size, size // 2, size // 2, size, size)
                self.assertTrue(expect_line in content)

        # test with `Mock` Hotspots
        configsgen.generate_static_cursor(
            imgs_dir=self.mock_images_path, out_dir=self.temp_dir, sizes=self.mock_sizes, hotspots=self.mock_hotspots)

        # testing .in config file
        # 📝 Note : Hard Coded test. size doesn't affect this test
        result_config_path = os.path.join(self.temp_dir, 'mock_static.in')
        with open(result_config_path, 'r') as file:
            content = file.read()

            for size in self.mock_sizes:
                expect_line: str = '28 2 2 28x28/mock_static.png'
                self.assertTrue(expect_line in content)

                expect_line: str = '50 3 3 50x50/mock_static.png'
                self.assertTrue(expect_line in content)

    def test_generate_animated_cursor(self):
        # test with `None` Hotspots
        configsgen.generate_animated_cursor(
            imgs_dir=self.mock_images_path, out_dir=self.temp_dir, sizes=self.mock_sizes, hotspots=None)

        # testing generated sized directory structure
        for size in self.mock_sizes:
            # have size direcory
            size_dir = '%sx%s' % (size, size)
            self.assertIn(size_dir, os.listdir(self.temp_dir))

            size_dir_path = os.path.join(self.temp_dir, size_dir)

            images_list: [str] = assets.get_mock_animated_images_list()

            gen_list = sorted(os.listdir(size_dir_path))

            # have resized images
            self.assertListEqual(images_list, gen_list)

            # test resized image dimensions & type as `.png`
            for image in images_list:
                img_path = os.path.join(size_dir_path, image)
                self.assert_image_size(img_path=img_path, size=size)

        # testing .in config file
        result_config_path = os.path.join(self.temp_dir, 'mock_animated_1.in')
        with open(result_config_path, 'r') as file:
            content = file.read()

            expect_line: str = '28 14 14 28x28/mock_animated_1-01.png 20\n28 14 14 28x28/mock_animated_1-02.png 20'
            self.assertTrue(expect_line in content)

        # test with `Mock` Hotspots
        configsgen.generate_animated_cursor(
            imgs_dir=self.mock_images_path, out_dir=self.temp_dir, sizes=self.mock_sizes, hotspots=self.mock_hotspots)

        # testing .in config file
        # 📝 Note : Hard Coded test. size doesn't affect this test
        result_config_path = os.path.join(self.temp_dir, 'mock_animated_1.in')
        with open(result_config_path, 'r') as file:
            content = file.read()

            expect_line: str = '28 2 2 28x28/mock_animated_1-01.png 20\n28 2 2 28x28/mock_animated_1-02.png 20\n50 3 3 50x50/mock_animated_1-01.png 20\n50 3 3 50x50/mock_animated_1-02.png 20'
            self.assertTrue(expect_line in content)


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
# encoding: utf-8

import io
import os
import sys
from argparse import Namespace
import shlex
import struct
import math

from PIL import Image
from PIL import ImageFilter

from .types import Path

p = struct.pack

program_name = 'anicursorgen'
program_version = '1.0.0'


def main(input_config: Path, output_file: Path, prefix: Path):
    """
        'win.py' is restrong of 'anicursorgen.py'.
        'input_config' is path to config_file.
        'output_file' is a path to store process cursor.
        In 'input_config' & 'output_file' absolute or relative both aceptable.
        'prefix' is a path to '.png files' link in the config_file, if relative path implemented.
        Generate .cur & .ani files based on '.in' config file.
        anicursorgen.py main copy found at <https://github.com/ubuntu/yaru/blob/master/icons/src/cursors/anicursorgen.py>
    """

    args = Namespace(add_shadows=False,
                     blur=3.125,
                     color='0x00000040',
                     down_shift=3.125,
                     input_config='-',
                     output_file='-',
                     prefix=None,
                     right_shift=9.375)

    if (input_config is not None):
        args.input_config = input_config
    if (output_file is not None):
        args.output_file = output_file
    if (prefix is not None):
        args.prefix = prefix

    try:
        if args.color[0] != '0' or args.color[1] not in [
                'x', 'X'
        ] or len(args.color) != 10:
            raise ValueError
        args.color = (int(args.color[2:4], 16), int(args.color[4:6], 16),
                      int(args.color[6:8], 16), int(args.color[8:10], 16))
    except:
        print("Can't parse the color '{}'".format(args.color), file=sys.stderr)
        return 1

    if args.prefix is None:
        args.prefix = os.getcwd()

    if args.input_config == '-':
        input_config = sys.stdin
    else:
        input_config = open(args.input_config, 'rb')

    if args.output_file == '-':
        output_file = sys.stdout
    else:
        output_file = open(args.output_file, 'wb')

    result = make_cursor_from(input_config, output_file, args)

    input_config.close()
    output_file.close()

    return result


def make_cursor_from(inp: Path, out: Path, args):
    frames = parse_config_from(inp, args.prefix)

    animated = frames_have_animation(frames)

    if animated:
        result = make_ani(frames, out, args)
    else:
        buf = make_cur(frames, args)
        copy_to(out, buf)
        result = 0

    return result


def copy_to(out: Path, buf):
    buf.seek(0, io.SEEK_SET)
    while True:
        b = buf.read(1024)
        if len(b) == 0:
            break
        out.write(b)


def frames_have_animation(frames):
    sizes = set()

    for frame in frames:
        if frame[4] == 0:
            continue
        if frame[0] in sizes:
            return True
        sizes.add(frame[0])

    return False


def make_cur(frames, args, animated=False):
    buf = io.BytesIO()
    buf.write(p('<HHH', 0, 2, len(frames)))
    frame_offsets = []

    frames = sorted(frames, reverse=True)

    for frame in frames:
        width = frame[0]
        if width > 255:
            width = 0
        height = width
        buf.write(p('<BBBB HH', width, height, 0, 0, frame[1], frame[2]))
        size_offset_pos = buf.seek(0, io.SEEK_CUR)
        buf.write(p('<II', 0, 0))
        frame_offsets.append([size_offset_pos])

    for i, frame in enumerate(frames):
        frame_offset = buf.seek(0, io.SEEK_CUR)
        frame_offsets[i].append(frame_offset)

        frame_png = Image.open(frame[3])

        if args.add_shadows:
            succeeded, shadowed = create_shadow(frame_png, args)
            if succeeded == 0:
                frame_png.close()
                frame_png = shadowed


#   Windows 10 fails to read PNG-compressed cursors for some reason
#   and the information about storing PNG-compressed cursors is
#   sparse. This is why PNG compression is not used.
#   Previously this was conditional on cursor size (<= 48 to be uncompressed).
        compressed = False

        #   On the other hand, Windows 10 refuses to read very large
        #   uncompressed animated cursor files, but does accept
        #   PNG-compressed animated cursors for some reason. Go figure.
        if animated:
            compressed = True

        if compressed:
            write_png(buf, frame, frame_png)
        else:
            write_cur(buf, frame, frame_png)

        frame_png.close()

        frame_end = buf.seek(0, io.SEEK_CUR)
        frame_offsets[i].append(frame_end - frame_offset)

    for frame_offset in frame_offsets:
        buf.seek(frame_offset[0])
        buf.write(p('<II', frame_offset[2], frame_offset[1]))

    return buf


def make_framesets(frames):
    framesets = []
    sizes = set()

    # This assumes that frames are sorted
    size = 0
    for i, frame in enumerate(frames):
        if size == 0 or frame[0] != size:
            size = frame[0]
            counter = 0

            if size in sizes:
                print(
                    "Frames are not sorted: frame {} has size {}, but we have seen that already"
                    .format(i, size),
                    file=sys.stderr)
                return None

            sizes.add(size)

        if counter >= len(framesets):
            framesets.append([])

        framesets[counter].append(frame)
        counter += 1

    for i in range(1, len(framesets)):
        if len(framesets[i - 1]) != len(framesets[i]):
            print("Frameset {} has size {}, expected {}".format(
                i, len(framesets[i]), len(framesets[i - 1])),
                file=sys.stderr)
            return None

    for frameset in framesets:
        for i in range(1, len(frameset)):
            if frameset[i - 1][4] != frameset[i][4]:
                print(
                    "Frameset {} has duration {} for framesize {}, but {} for framesize {}"
                    .format(i, frameset[i][4], frameset[i][0],
                            frameset[i - 1][4], frameset[i - 1][0]),
                    file=sys.stderr)
                return None
    framesets = sorted(framesets, reverse=True)

    return framesets


def make_ani(frames, out: Path, args):
    framesets = make_framesets(frames)
    if framesets is None:
        return 1

    buf = io.BytesIO()

    buf.write(b'RIFF')
    riff_len_pos = buf.seek(0, io.SEEK_CUR)
    buf.write(p('<I', 0))
    riff_len_start = buf.seek(0, io.SEEK_CUR)

    buf.write(b'ACON')
    buf.write(b'anih')
    buf.write(
        p('<IIIIIIIIII', 36, 36, len(framesets), len(framesets), 0, 0, 32, 1,
          framesets[0][0][4], 0x01))

    rates = set()
    for frameset in framesets:
        rates.add(frameset[0][4])

    if len(rates) != 1:
        buf.write(b'rate')
        buf.write(p('<I', len(framesets) * 4))
        for frameset in framesets:
            buf.write(p('<I', frameset[0][4]))

    buf.write(b'LIST')
    list_len_pos = buf.seek(0, io.SEEK_CUR)
    buf.write(p('<I', 0))
    list_len_start = buf.seek(0, io.SEEK_CUR)

    buf.write(b'fram')

    for frameset in framesets:
        buf.write(b'icon')
        cur = make_cur(frameset, args, animated=True)
        cur_size = cur.seek(0, io.SEEK_END)
        # aligned_cur_size = cur_size
        # if cur_size % 4 != 0:
        #  aligned_cur_size += 4 - cur_size % 2
        buf.write(p('<i', cur_size))
        copy_to(buf, cur)
        pos = buf.seek(0, io.SEEK_END)
        if pos % 2 != 0:
            buf.write(('\x00' * (2 - (pos % 2))).encode())

    end_at = buf.seek(0, io.SEEK_CUR)
    buf.seek(riff_len_pos, io.SEEK_SET)
    buf.write(p('<I', end_at - riff_len_start))
    buf.seek(list_len_pos, io.SEEK_SET)
    buf.write(p('<I', end_at - list_len_start))

    copy_to(out, buf)

    return 0


def write_png(out: Path, frame, frame_png):
    frame_png.save(out, "png", optimize=True)


def write_cur(out: Path, frame, frame_png):
    pixels = frame_png.load()

    out.write(
        p('<I II HH IIIIII', 40, frame[0], frame[0] * 2, 1, 32, 0, 0, 0, 0, 0,
          0))

    for y in reversed(list(range(frame[0]))):
        for x in range(frame[0]):
            pixel = pixels[x, y]
            out.write(p('<BBBB', pixel[2], pixel[1], pixel[0], pixel[3]))

    acc = 0
    acc_pos = 0
    for y in reversed(list(range(frame[0]))):
        wrote = 0
        for x in range(frame[0]):
            if pixels[x, y][3] <= 127:
                acc = acc | (1 << acc_pos)
            acc_pos += 1
            if acc_pos == 8:
                acc_pos = 0
                out.write(chr(acc).encode())
                wrote += 1
        if wrote % 4 != 0:
            out.write(b'\x00' * (4 - wrote % 4))


def parse_config_from(inp, prefix: Path):
    frames = []
    for line in inp.readlines():
        line = line.decode()
        words = shlex.split(line.rstrip('\n').rstrip('\r'))
        # print(words)
        if len(words) < 4:
            continue

        try:
            size = int(words[0])
            # print(size)
            hotx = int(words[1]) - 1
            # print(hotx)
            hoty = int(words[2]) - 1
            # print(hoty)
            filename = words[3]
            # print(filename)
            if not os.path.isabs(filename):
                filename = prefix + '/' + filename
        except:
            continue

        if len(words) > 4:
            try:
                duration = int(words[4])
            except:
                continue
        else:
            duration = 0

        frames.append((size, hotx, hoty, filename, duration))

    return frames


def create_shadow(orig, args):
    blur_px = orig.size[0] / 100.0 * args.blur
    right_px = int(orig.size[0] / 100.0 * args.right_shift)
    down_px = int(orig.size[1] / 100.0 * args.down_shift)

    shadow = Image.new('RGBA', orig.size, (0, 0, 0, 0))
    shadowize(shadow, orig, args.color)
    shadow.load()

    if args.blur > 0:
        crop = (int(math.floor(-blur_px)), int(math.floor(-blur_px)),
                orig.size[0] + int(math.ceil(blur_px)),
                orig.size[1] + int(math.ceil(blur_px)))
        right_px += int(math.floor(-blur_px))
        down_px += int(math.floor(-blur_px))
        shadow = shadow.crop(crop)
        flt = ImageFilter.GaussianBlur(blur_px)
        shadow = shadow.filter(flt)
    shadow.load()

    shadowed = Image.new('RGBA', orig.size, (0, 0, 0, 0))
    shadowed.paste(shadow, (right_px, down_px))
    shadowed.crop((0, 0, orig.size[0], orig.size[1]))
    shadowed = Image.alpha_composite(shadowed, orig)

    return 0, shadowed


def shadowize(shadow, orig, color):
    o_pxs = orig.load()
    s_pxs = shadow.load()
    for y in range(orig.size[1]):
        for x in range(orig.size[0]):
            o_px = o_pxs[x, y]
            if o_px[3] > 0:
                s_pxs[x, y] = (color[0], color[1], color[2],
                               int(color[3] * (o_px[3] / 255.0)))



#!/usr/bin/env python
# encoding: utf-8

import json
import os

from .__main__ import main as build

from .x11 import main as x11_gen

from .win import main as win_gen

from .linker.__main__ import link_cursors
from .linker import linker

from .template.__main__ import create_x11_template
from .template import template

from .configsgen.__main__ import generate_animated_cursor, generate_static_cursor
from .configsgen import configsgen

from .build import build_cursor_theme, build_win_cursor_theme, build_x11_cursor_theme

with open(os.path.join(os.path.dirname(__file__), 'pkginfo.json')) as fp:
    _info = json.load(fp)

__version__ = _info['version']
info = _info



#!/usr/bin/env python
# encoding: utf-8

import json
import subprocess
from setuptools import setup, find_namespace_packages
from distutils.command.install import install as _install

try:
    # for pip >= 10
    from pip._internal.req import parse_requirements
except ImportError:
    # for pip <= 9.0.3
    from pip.req import parse_requirements


# third-party dependencies
def load_requirements(fname):
    reqs = parse_requirements(fname, session='clickgen_session')
    return [str(ir.requirement) for ir in reqs]


class install(_install):
    def run(self):
        subprocess.call(['make', 'clean', '-C', 'xcursorgen'])
        subprocess.call(['make', '-C', 'xcursorgen'])
        _install.run(self)


# readme.md as long description
with open("README.md", "r") as fh:
    long_description = fh.read()

with open('clickgen/pkginfo.json') as fp:
    _info = json.load(fp)

setup(
    name=_info['name'],
    version=_info['version'],
    author=_info['author'],
    author_email=_info['author_email'],
    description=_info['description'],
    long_description=long_description,
    long_description_content_type="text/markdown",
    url='https://github.com/KaizIqbal/clickgen',
    classifiers=[
        _info['status_classifier'], 'Topic :: System :: Operating System',
        'Programming Language :: Python :: 3', 'Programming Language :: C',
        'Natural Language :: English',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent', 'Typing :: Typed'],
    cmdclass={'install': install},
    python_requires='>=3.6',
    scripts=['scripts/clickgen'],
    keywords=['cursor', 'xcursor', 'windows',
              'linux', 'anicursorgen', 'xcursorgen'],
    install_requires=load_requirements('requirements.txt'),
    packages=find_namespace_packages(include=['clickgen', 'clickgen.*']),
    include_package_data=True,
    zip_safe=True
)



#!/usr/bin/env python
# encoding: utf-8

from difflib import SequenceMatcher as SM
from pathlib import Path
import itertools
import json
import os

from ..helpers import cd, get_logger, symlink
from ..types import Path, StringList, Logger

# cursor data paths
basedir: Path = os.path.abspath(os.path.dirname(__file__))
data_file: Path = os.path.join(basedir, 'data.json')

# Logger
logger: Logger = get_logger('clickgen:linker')


def match_to_directory(name: str, directory: StringList) -> str:
    """
        Match `name` with `directory` with 0.5 ratio. That can fix `name` typo by searching inside directory.
        Example:
            z00m => zoom
    """

    compare_ratio = 0.5
    match = name

    for word in directory:
        ratio = SM(None, name.lower(), word.lower()).ratio()
        if ratio > compare_ratio:
            compare_ratio = ratio
            match = word

    logger.info('"%s" match to "%s" with "%s" ratio' %
                (name, match, compare_ratio))

    return match


def load_data() -> [StringList]:
    """
        Load ./data.json file and return processed two lists.i.e.:cursors, all_cursors
        `cursors` is `group` of similar cursors, Means one cursor from them satisfied all other cursors by generating the symbolic link.
        `all` is used as a `directory` where all cursors return as a list without any `group`.
    """
    with open(data_file) as f:
        data = json.loads(f.read())

    cursors = data['cursors']
    all = list(itertools.chain.from_iterable(cursors))

    return cursors, all


def link_cursors(directory: Path, win: bool = False) -> None:
    """
        Generate missing cursors have similar endpoint inside `directory`.
        `win` flag is `False` default, If it `True` this function only fix the name of cursors because `Windows` does not support symblinks.
    """
    directory = os.path.abspath(directory)
    isExists = os.path.exists(directory)

    # user have cursors fot symblink
    cursors = []

    try:
        if not isExists:
            raise FileNotFoundError('x11 directory not found')

        for file in os.listdir(directory):
            cursors.append(file)

        if (len(cursors) <= 0):
            raise FileNotFoundError('directory is empty')

    except FileNotFoundError as err:
        print('Error: ', err)

    known_cursors, all_cursors = load_data()

    # rename cursor with proper name
    for index, cursor in enumerate(cursors):
        cursor, extension = os.path.splitext(cursor)
        fix_cur = match_to_directory(cursor, all_cursors)

        if fix_cur not in all_cursors:
            msg = '%s is unknown cursor' % fix_cur
            logger.warning(msg)
            print(msg)

        elif (fix_cur != cursor):
            old_path = os.path.join(directory, cursor + extension)
            new_path = os.path.join(directory, fix_cur + extension)
            os.rename(old_path, new_path)

            cursors[index] = fix_cur

            msg = 'Fixed: %s ==> %s' % (cursor, fix_cur)
            logger.info(msg)
            print(msg)

    # symbolic links only for x11
    if not win:
        # cd for relative links
        with cd(directory):
            for cursor in cursors:
                for relative in known_cursors:
                    if cursor in relative:
                        # remove source cursor
                        relative.remove(cursor)

                        # links to other if not empty
                        if len(relative) != 0:
                            for link in relative:
                                src = './' + cursor
                                dst = './' + link
                                symlink(src, dst, overwrite=True)
                            print('Symbolic link: %s ==> ' % (cursor),
                                  *relative)



#!/usr/bin/env python
# encoding: utf-8

from contextlib import contextmanager
import logging
import os
import shutil
import sys
import tempfile

from . import color_logger
from .types import Path


def get_logger(name: str) -> logging.Logger:
    """
        To get custom `logging` instance.
        Return a logger with the specified name, creating it if necessary.
        If no name is specified, return the root logger.
    """

    logger = logging.getLogger(name)
    logging.basicConfig(level=logging.INFO)

    return logger


logger = get_logger('clickgen:helpers')


def create_dir(path: Path) -> None:
    """
        ⚡ Clickgen Helper Function ⚡

        Create a directory if not exists.
        'path' is an absolute or relative path to the directory to create.
    """

    if not os.path.exists(path):
        os.makedirs(path)


@contextmanager
def TemporaryDirectory():
    """
        ⚡ Clickgen Helper Function ⚡

        Work with Temporary Directory.
        Use this context with `with` syntax as following:

            with TemporaryDirectory() as temp_dir:
                ...

    """
    name = tempfile.mkdtemp()
    try:
        yield name
        logger.info('Entering to %s' % name)
    finally:
        shutil.rmtree(name)
        logger.info('Exist and remove %s' % name)


@contextmanager
def cd(path: Path):
    """
        ⚡ Clickgen Helper Function ⚡

        Temporary change directory.
        Use this context with `with` syntax as following:

            with cd('/tmp'):
                ...
    """

    CWD = os.getcwd()

    os.chdir(path)
    try:
        yield
        logger.info('Change directory to %s' % path)
    except:
        print('Exception caught: %s' % sys.exc_info()[0])
    finally:
        os.chdir(CWD)
        logger.info('Back to Normal as %s' % CWD)


def symlink(target: Path, link_name: Path, overwrite=False):
    """
        ⚡ Clickgen Helper Function ⚡

        Create a symbolic link named 'link_name' pointing to 'target'.
        If link_name exists then FileExistsError is raised, unless 'overwrite=True'.
        When trying to overwrite a directory, IsADirectoryError is raised.

        ref => https://stackoverflow.com/a/55742015
    """

    if not overwrite:
        os.symlink(target, link_name)
        return

    link_dir = os.path.dirname(link_name)

    while True:
        temp_link_name = tempfile.mktemp(dir=link_dir)

        try:
            os.symlink(target, temp_link_name)
            break
        except FileExistsError:
            pass

    try:
        if os.path.isdir(link_name):
            raise IsADirectoryError(
                f"Cannot symlink over existing directory: '{link_name}'")
        os.replace(temp_link_name, link_name)
    except:
        if os.path.islink(temp_link_name):
            os.remove(temp_link_name)
        raise



#!/usr/bin/env python
# encoding: utf-8

import os
import logging
import tempfile

import unittest
from unittest.mock import patch

from clickgen import helpers


class TestHelpesMethods(unittest.TestCase):

    # setup
    def setUp(self):
        logging.disable(logging.CRITICAL)

        self.orig_cwd = os.getcwd()
        self.dst_dir = 'foo'

        self.test_target = os.path.join(tempfile.gettempdir(), 'foo')
        self.test_link_name = os.path.join(tempfile.gettempdir(), 'bar')

    def tearDown(self):
        logging.disable(logging.NOTSET)

    # tests
    def test_get_looger(self):
        foo_logger = helpers.get_logger('foo')
        self.assertEqual(logging.getLogger('foo'), foo_logger)

    @patch('clickgen.helpers.os.makedirs')
    @patch('clickgen.helpers.os.path.exists')
    def test_create_dir(self, mock_make_dirs, mock_exists):
        mock_exists.return_value = True
        helpers.create_dir('/foo/bar')
        mock_make_dirs.assert_called_with('/foo/bar')

    def test_TemporaryDirectory(self):
        with helpers.TemporaryDirectory() as mock_tmp_dir:
            self.assertTrue(os.path.isdir(mock_tmp_dir))

    @patch('clickgen.helpers.os.chdir')
    def test_cd(self, mock_chdir):
        with helpers.cd(self.dst_dir):
            mock_chdir.reset_mock()
        mock_chdir.assert_called_once_with(self.orig_cwd)

    @patch('clickgen.helpers.os.replace')
    @patch('clickgen.helpers.tempfile.mktemp')
    @patch('clickgen.helpers.os.symlink')
    def test_symblink(self, mock_symlink, mock_temp_link, mock_replace):

        # testing general symbolic link
        helpers.symlink(target=self.test_target,
                        link_name=self.test_link_name,
                        overwrite=False)
        mock_symlink.assert_called_with(self.test_target, self.test_link_name)

        # testing force symbolic link
        tmp_link = '/asdasd'
        mock_temp_link.return_value = tmp_link

        helpers.symlink(target=self.test_target,
                        link_name=self.test_link_name,
                        overwrite=True)
        mock_symlink.assert_called_with(self.test_target, tmp_link)
        mock_replace.assert_called_with(tmp_link, self.test_link_name)


if __name__ == '__main__':
    unittest.main()
